@@ -116,6 +116,20 @@ static void msg_delivered (rd_kafka_t *rk,
 		fprintf(stderr, "%% Message delivered (%zd bytes)\n", len);
 }
 
+/**
+ * Message delivery report callback using the richer rd_kafka_message_t object.
+ */
+static void msg_delivered2 (rd_kafka_t *rk,
+                            const rd_kafka_message_t *rkmessage, void *opaque) {
+        if (rkmessage->err)
+		fprintf(stderr, "%% Message delivery failed: %s\n",
+                        rd_kafka_message_errstr(rkmessage));
+	else if (!quiet)
+		fprintf(stderr,
+                        "%% Message delivered (%zd bytes, offset %"PRId64")\n",
+                        rkmessage->len, rkmessage->offset);
+}
+
 
 static void msg_consume (rd_kafka_message_t *rkmessage,
 			 void *opaque) {
@@ -164,6 +178,65 @@ static void msg_consume (rd_kafka_message_t *rkmessage,
 }
 
 
+static void metadata_print (const char *topic,
+                            const struct rd_kafka_metadata *metadata) {
+        int i, j, k;
+
+        printf("Metadata for %s (from broker %"PRId32": %s):\n",
+               topic ? : "all topics",
+               metadata->orig_broker_id,
+               metadata->orig_broker_name);
+
+
+        /* Iterate brokers */
+        printf(" %i brokers:\n", metadata->broker_cnt);
+        for (i = 0 ; i < metadata->broker_cnt ; i++)
+                printf("  broker %"PRId32" at %s:%i\n",
+                       metadata->brokers[i].id,
+                       metadata->brokers[i].host,
+                       metadata->brokers[i].port);
+
+        /* Iterate topics */
+        printf(" %i topics:\n", metadata->topic_cnt);
+        for (i = 0 ; i < metadata->topic_cnt ; i++) {
+                const struct rd_kafka_metadata_topic *t = &metadata->topics[i];
+                printf("  topic \"%s\" with %i partitions:",
+                       t->topic,
+                       t->partition_cnt);
+                if (t->err) {
+                        printf(" %s", rd_kafka_err2str(t->err));
+                        if (t->err == RD_KAFKA_RESP_ERR_LEADER_NOT_AVAILABLE)
+                                printf(" (try again)");
+                }
+                printf("\n");
+
+                /* Iterate topic's partitions */
+                for (j = 0 ; j < t->partition_cnt ; j++) {
+                        const struct rd_kafka_metadata_partition *p;
+                        p = &t->partitions[j];
+                        printf("    partition %"PRId32", "
+                               "leader %"PRId32", replicas: ",
+                               p->id, p->leader);
+
+                        /* Iterate partition's replicas */
+                        for (k = 0 ; k < p->replica_cnt ; k++)
+                                printf("%s%"PRId32,
+                                       k > 0 ? ",":"", p->replicas[k]);
+
+                        /* Iterate partition's ISRs */
+                        printf(", isrs: ");
+                        for (k = 0 ; k < p->isr_cnt ; k++)
+                                printf("%s%"PRId32,
+                                       k > 0 ? ",":"", p->isrs[k]);
+                        if (p->err)
+                                printf(", %s\n", rd_kafka_err2str(p->err));
+                        else
+                                printf("\n");
+                }
+        }
+}
+
+
 static void sig_usr1 (int sig) {
 	rd_kafka_dump(stdout, rk);
 }
@@ -180,6 +253,7 @@ int main (int argc, char **argv) {
 	char errstr[512];
 	const char *debug = NULL;
 	int64_t start_offset = 0;
+        int report_offsets = 0;
 	int do_conf_dump = 0;
 
 	quiet = !isatty(STDIN_FILENO);
@@ -190,10 +264,11 @@ int main (int argc, char **argv) {
 	/* Topic configuration */
 	topic_conf = rd_kafka_topic_conf_new();
 
-	while ((opt = getopt(argc, argv, "PCt:p:b:z:qd:o:eX:A")) != -1) {
+	while ((opt = getopt(argc, argv, "PCLt:p:b:z:qd:o:eX:A")) != -1) {
 		switch (opt) {
 		case 'P':
 		case 'C':
+                case 'L':
 			mode = opt;
 			break;
 		case 't':
@@ -221,8 +296,14 @@ int main (int argc, char **argv) {
 				start_offset = RD_KAFKA_OFFSET_BEGINNING;
 			else if (!strcmp(optarg, "stored"))
 				start_offset = RD_KAFKA_OFFSET_STORED;
-			else
+                        else if (!strcmp(optarg, "report"))
+                                report_offsets = 1;
+			else {
 				start_offset = strtoll(optarg, NULL, 10);
+
+				if (start_offset < 0)
+					start_offset = RD_KAFKA_OFFSET_TAIL(-start_offset);
+			}
 			break;
 		case 'e':
 			exit_eof = 1;
@@ -321,22 +402,25 @@ int main (int argc, char **argv) {
 	}
 
 
-	if (!topic || optind != argc) {
+	if (optind != argc || (mode != 'L' && !topic)) {
 	usage:
 		fprintf(stderr,
-			"Usage: %s [-C|-P] -t <topic> "
+			"Usage: %s -C|-P|-L -t <topic> "
 			"[-p <partition>] [-b <host1:port1,host2:port2,..>]\n"
 			"\n"
 			"librdkafka version %s (0x%08x)\n"
 			"\n"
 			" Options:\n"
 			"  -C | -P         Consumer or Producer mode\n"
+                        "  -L              Metadata list mode\n"
 			"  -t <topic>      Topic to fetch / produce\n"
 			"  -p <num>        Partition (random partitioner)\n"
 			"  -b <brokers>    Broker address (localhost:9092)\n"
 			"  -z <codec>      Enable compression:\n"
 			"                  none|gzip|snappy\n"
-			"  -o <offset>     Start offset (consumer)\n"
+			"  -o <offset>     Start offset (consumer):\n"
+			"                  beginning, end, NNNNN or -NNNNN\n"
+                        "  -o report       Report message offsets (producer)\n"
 			"  -e              Exit consumer when last message\n"
 			"                  in partition has been received.\n"
 			"  -d [facs..]     Enable debugging contexts:\n"
@@ -354,6 +438,9 @@ int main (int argc, char **argv) {
 			"  writes fetched messages to stdout\n"
 			" In Producer mode:\n"
 			"  reads messages from stdin and sends to broker\n"
+                        " In List mode:\n"
+                        "  queries broker for metadata information, "
+                        "topic is optional.\n"
 			"\n"
 			"\n"
 			"\n",
@@ -385,7 +472,16 @@ int main (int argc, char **argv) {
 		/* Set up a message delivery report callback.
 		 * It will be called once for each message, either on successful
 		 * delivery to broker, or upon failure to deliver to broker. */
-		rd_kafka_conf_set_dr_cb(conf, msg_delivered);
+
+                /* If offset reporting (-o report) is enabled, use the
+                 * richer dr_msg_cb instead. */
+                if (report_offsets) {
+                        rd_kafka_topic_conf_set(topic_conf,
+                                                "produce.offset.report",
+                                                "true", errstr, sizeof(errstr));
+                        rd_kafka_conf_set_dr_msg_cb(conf, msg_delivered2);
+                } else
+                        rd_kafka_conf_set_dr_cb(conf, msg_delivered);
 
 		/* Create Kafka handle */
 		if (!(rk = rd_kafka_new(RD_KAFKA_PRODUCER, conf,
@@ -456,6 +552,9 @@ int main (int argc, char **argv) {
 		while (run && rd_kafka_outq_len(rk) > 0)
 			rd_kafka_poll(rk, 100);
 
+		/* Destroy topic */
+		rd_kafka_topic_destroy(rkt);
+
 		/* Destroy the handle */
 		rd_kafka_destroy(rk);
 
@@ -512,10 +611,71 @@ int main (int argc, char **argv) {
 		/* Stop consuming */
 		rd_kafka_consume_stop(rkt, partition);
 
+		/* Destroy topic */
 		rd_kafka_topic_destroy(rkt);
 
+		/* Destroy handle */
 		rd_kafka_destroy(rk);
-	}
+
+        } else if (mode == 'L') {
+                rd_kafka_resp_err_t err = RD_KAFKA_RESP_ERR_NO_ERROR;
+
+		/* Create Kafka handle */
+		if (!(rk = rd_kafka_new(RD_KAFKA_PRODUCER, conf,
+					errstr, sizeof(errstr)))) {
+			fprintf(stderr,
+				"%% Failed to create new producer: %s\n",
+				errstr);
+			exit(1);
+		}
+
+		/* Set logger */
+		rd_kafka_set_logger(rk, logger);
+		rd_kafka_set_log_level(rk, LOG_DEBUG);
+
+		/* Add brokers */
+		if (rd_kafka_brokers_add(rk, brokers) == 0) {
+			fprintf(stderr, "%% No valid brokers specified\n");
+			exit(1);
+		}
+
+                /* Create topic */
+                if (topic)
+                        rkt = rd_kafka_topic_new(rk, topic, topic_conf);
+                else
+                        rkt = NULL;
+
+                while (run) {
+                        const struct rd_kafka_metadata *metadata;
+
+                        /* Fetch metadata */
+                        err = rd_kafka_metadata(rk, rkt ? 0 : 1, rkt,
+                                                &metadata, 5000);
+                        if (err != RD_KAFKA_RESP_ERR_NO_ERROR) {
+                                fprintf(stderr,
+                                        "%% Failed to acquire metadata: %s\n",
+                                        rd_kafka_err2str(err));
+                                run = 0;
+                                break;
+                        }
+
+                        metadata_print(topic, metadata);
+
+                        rd_kafka_metadata_destroy(metadata);
+                        run = 0;
+                }
+
+		/* Destroy topic */
+		if (rkt)
+			rd_kafka_topic_destroy(rkt);
+
+		/* Destroy the handle */
+		rd_kafka_destroy(rk);
+
+                /* Exit right away, dont wait for background cleanup, we haven't
+                 * done anything important anyway. */
+                exit(err ? 2 : 0);
+        }
 
 	/* Let background threads clean up and terminate cleanly. */
 	rd_kafka_wait_destroyed(2000);
